@@ -5,6 +5,7 @@ from tqdm import trange, tqdm
 from urllib.parse import urlparse
 import tempfile
 import cv2
+from scipy.stats import mode
 from . import transforms, dynamics, utils, plot, metrics
 
 try:
@@ -394,7 +395,7 @@ class UnetModel():
 
         """
         if isinstance(self.pretrained_model, str) or not net_avg:  
-            y, style = self._run_net(img, augment=augment, tile=tile, 
+            y, style = self._run_net(img, augment=augment, tile=tile, tile_overlap=tile_overlap,
                                      bsize=bsize, return_conv=return_conv)
         else:  
             for j in range(len(self.pretrained_model)):
@@ -842,14 +843,36 @@ class UnetModel():
               test_data=None, test_labels=None,
               save_path=None, save_every=100, save_each=False,
               learning_rate=0.2, n_epochs=500, momentum=0.9, weight_decay=0.00001, 
-              SGD=True, batch_size=8, rescale=True, netstr=None): 
+              SGD=True, batch_size=8, nimg_per_epoch=None, rescale=True, netstr=None): 
         """ train function uses loss function self.loss_fn in models.py"""
         
         d = datetime.datetime.now()
-        self.learning_rate = learning_rate
+        
         self.n_epochs = n_epochs
+        if isinstance(learning_rate, (list, np.ndarray)):
+            if isinstance(learning_rate, np.ndarray) and learning_rate.ndim > 1:
+                raise ValueError('learning_rate.ndim must equal 1')
+            elif len(learning_rate) != n_epochs:
+                raise ValueError('if learning_rate given as list or np.ndarray it must have length n_epochs')
+            self.learning_rate = learning_rate
+            self.learning_rate_const = mode(learning_rate)[0][0]
+        else:
+            self.learning_rate_const = learning_rate
+            # set learning rate schedule    
+            if SGD:
+                LR = np.linspace(0, self.learning_rate_const, 10)
+                if self.n_epochs > 250:
+                    LR = np.append(LR, self.learning_rate_const*np.ones(self.n_epochs-100))
+                    for i in range(10):
+                        LR = np.append(LR, LR[-1]/2 * np.ones(10))
+                else:
+                    LR = np.append(LR, self.learning_rate_const*np.ones(max(0,self.n_epochs-10)))
+            else:
+                LR = self.learning_rate_const * np.ones(self.n_epochs)
+            self.learning_rate = LR
+
         self.batch_size = batch_size
-        self._set_optimizer(self.learning_rate, momentum, weight_decay, SGD)
+        self._set_optimizer(self.learning_rate[0], momentum, weight_decay, SGD)
         self._set_criterion()
         
         nimg = len(train_data)
@@ -868,7 +891,7 @@ class UnetModel():
 
         nchan = train_data[0].shape[0]
         core_logger.info('>>>> training network with %d channel input <<<<'%nchan)
-        core_logger.info('>>>> LR: %0.5f, batch_size: %d, weight_decay: %0.5f'%(self.learning_rate, self.batch_size, weight_decay))
+        core_logger.info('>>>> LR: %0.5f, batch_size: %d, weight_decay: %0.5f'%(self.learning_rate_const, self.batch_size, weight_decay))
         
         if test_data is not None:
             core_logger.info(f'>>>> ntrain = {nimg}, ntest = {len(test_data)}')
@@ -877,17 +900,7 @@ class UnetModel():
         
         tic = time.time()
 
-        # set learning rate schedule    
-        if SGD:
-            LR = np.linspace(0, self.learning_rate, 10)
-            if self.n_epochs > 250:
-                LR = np.append(LR, self.learning_rate*np.ones(self.n_epochs-100))
-                for i in range(10):
-                    LR = np.append(LR, LR[-1]/2 * np.ones(10))
-            else:
-                LR = np.append(LR, self.learning_rate*np.ones(max(0,self.n_epochs-10)))
-        else:
-            LR = self.learning_rate * np.ones(self.n_epochs)
+        
 
         lavg, nsum = 0, 0
 
@@ -906,17 +919,22 @@ class UnetModel():
         # cannot train with mkldnn
         self.net.mkldnn = False
 
-        for iepoch in range(self.n_epochs):
-                
-            np.random.seed(iepoch)
-            if nimg < batch_size:
-                rperm = np.random.choice(nimg, batch_size)
-            else:
-                rperm = np.random.permutation(nimg)
+        # get indices for each epoch for training
+        np.random.seed(0)
+        inds_all = np.zeros((0,), 'int32')
+        if nimg_per_epoch is None or nimg > nimg_per_epoch:
+            nimg_per_epoch = nimg 
+        core_logger.info(f'>>>> nimg_per_epoch = {nimg_per_epoch}')
+        while len(inds_all) < n_epochs * nimg_per_epoch:
+            rperm = np.random.permutation(nimg)
+            inds_all = np.hstack((inds_all, rperm))
+        
+        for iepoch in range(self.n_epochs):    
             if SGD:
-                self._set_learning_rate(LR[iepoch])
-            
-            for ibatch in range(0,nimg,batch_size):
+                self._set_learning_rate(self.learning_rate[iepoch])
+            np.random.seed(iepoch)
+            rperm = inds_all[iepoch*nimg_per_epoch:(iepoch+1)*nimg_per_epoch]
+            for ibatch in range(0,nimg_per_epoch,batch_size):
                 inds = rperm[ibatch:ibatch+batch_size]
                 rsc = diam_train[inds] / self.diam_mean if rescale else np.ones(len(inds), np.float32)
                 # now passing in the full train array, need the labels for distance field
@@ -929,7 +947,7 @@ class UnetModel():
                 lavg += train_loss
                 nsum += len(imgi) 
             
-            if iepoch%10==0 or iepoch<10:
+            if iepoch%10==0 or iepoch==5:
                 lavg = lavg / nsum
                 if test_data is not None:
                     lavgt, nsum = 0., 0
@@ -949,10 +967,10 @@ class UnetModel():
                         nsum += len(imgi)
 
                     core_logger.info('Epoch %d, Time %4.1fs, Loss %2.4f, Loss Test %2.4f, LR %2.4f'%
-                            (iepoch, time.time()-tic, lavg, lavgt/nsum, LR[iepoch]))
+                            (iepoch, time.time()-tic, lavg, lavgt/nsum, self.learning_rate[iepoch]))
                 else:
                     core_logger.info('Epoch %d, Time %4.1fs, Loss %2.4f, LR %2.4f'%
-                            (iepoch, time.time()-tic, lavg, LR[iepoch]))
+                            (iepoch, time.time()-tic, lavg, self.learning_rate[iepoch]))
                 
                 lavg, nsum = 0, 0
                             
