@@ -14,7 +14,7 @@ from scipy.ndimage import gaussian_filter
 
 from . import guiparts, menus, io
 from .. import models, core, dynamics
-from ..utils import download_url_to_file, masks_to_outlines
+from ..utils import download_url_to_file, masks_to_outlines, diameters 
 from ..io import OMNI_INSTALLED, save_server, get_image_files, imsave, imread
 from ..transforms import resize_image, normalize99 #fixed import
 from ..plot import disk
@@ -33,6 +33,11 @@ try:
 except:
     SERVER_UPLOAD = False
 
+try:
+    import omnipose 
+except:
+    0
+
 #Define possible models; can we make a master list in another file to use in models and main? 
     
 class QHLine(QFrame):
@@ -177,7 +182,7 @@ class MainW(QMainWindow):
         menus.mainmenu(self)
         menus.editmenu(self)
         #menus.modelmenu(self)
-        self.model_strings = models.MODEL_NAMES
+        self.model_strings = models.MODEL_NAMES.copy()
         menus.helpmenu(self)
         if OMNI_INSTALLED:
             menus.omnimenu(self)
@@ -292,10 +297,10 @@ class MainW(QMainWindow):
         self.color = 0 # 0=RGB, 1=gray, 2=R, 3=G, 4=B
         self.RGBChoose = guiparts.RGBRadioButtons(self, b,4)
         self.RGBDropDown = QComboBox()
-        self.RGBDropDown.addItems(["RGB","gray","spectral","red","green","blue"])
+        self.RGBDropDown.addItems(["RGB","red=R","green=G","blue=B","gray","spectral"])
         self.RGBDropDown.setFont(self.medfont)
         self.RGBDropDown.currentIndexChanged.connect(self.color_choose)
-        self.RGBDropDown.setFixedWidth(60)
+        #self.RGBDropDown.setFixedWidth(60)
         self.RGBDropDown.setStyleSheet(self.dropdowns)
 
         self.l0.addWidget(self.RGBDropDown, b,0,1,4)
@@ -435,9 +440,9 @@ class MainW(QMainWindow):
 
         # fast mode
         self.NetAvg = QComboBox()
-        self.NetAvg.addItems(['average 4 nets', '+ resample (slow)', 'run 1 net (fast)', ])
+        self.NetAvg.addItems(['average 4 nets', 'run 1 net', '+ turn off resample (fast)'])
         self.NetAvg.setFont(self.medfont)
-        self.NetAvg.setToolTip('average 4 different fit networks (default) + resample for smooth masks (slow) or run 1 network (fast)')
+        self.NetAvg.setToolTip('average 4 different fit networks (default); run 1 network (faster); or run 1 net + turn off resample (fast)')
         self.l0.addWidget(self.NetAvg, b,4,1,4)
 
         b+=1
@@ -445,10 +450,11 @@ class MainW(QMainWindow):
         self.ModelChoose = QComboBox()
         if len(self.model_strings) > len(models.MODEL_NAMES):
             current_index = len(models.MODEL_NAMES)
+            self.NetAvg.setCurrentIndex(1)
         else:
             current_index = 0
         self.ModelChoose.addItems(self.model_strings) #added omnipose model names
-        self.ModelChoose.setFixedWidth(150)
+        self.ModelChoose.setFixedWidth(175)
         self.ModelChoose.setStyleSheet(self.dropdowns)
         self.ModelChoose.setFont(self.medfont)
         self.ModelChoose.setCurrentIndex(current_index)
@@ -561,6 +567,14 @@ class MainW(QMainWindow):
         label.setFont(self.boldfont)
         self.l0.addWidget(label, b,0,1,8)
 
+        #b+=1
+        #self.autochannelbtn = QCheckBox('renormalize channels')
+        #self.autochannelbtn.setStyleSheet(self.checkstyle)
+        #self.autochannelbtn.setFont(self.medfont)
+        #self.autochannelbtn.setChecked(True)
+        #self.autochannelbtn.setToolTip('sets channels so that 1st and 99th percentiles at same values, only works for 2D images currently')
+        #self.l0.addWidget(self.autochannelbtn, b,0,1,4)
+
         b+=1
         self.autobtn = QCheckBox('auto-adjust')
         self.autobtn.setStyleSheet(self.checkstyle)
@@ -568,6 +582,7 @@ class MainW(QMainWindow):
         self.autobtn.setChecked(True)
         self.l0.addWidget(self.autobtn, b,0,1,4)
 
+        
         b+=1
         self.slider = guiparts.RangeSlider(self)
         self.slider.setMinimum(0)
@@ -675,6 +690,24 @@ class MainW(QMainWindow):
                 elif event.key() == QtCore.Qt.Key_Down or event.key() == QtCore.Qt.Key_S:
                     self.color = (self.color+1)%(6)
                     self.RGBDropDown.setCurrentIndex(self.color)
+                elif event.key() == QtCore.Qt.Key_R:
+                    if self.color!=1:
+                        self.color = 1
+                    else:
+                        self.color = 0
+                    self.RGBDropDown.setCurrentIndex(self.color)
+                elif event.key() == QtCore.Qt.Key_G:
+                    if self.color!=2:
+                        self.color = 2
+                    else:
+                        self.color = 0
+                    self.RGBDropDown.setCurrentIndex(self.color)
+                elif event.key() == QtCore.Qt.Key_B:
+                    if self.color!=3:
+                        self.color = 3
+                    else:
+                        self.color = 0
+                    self.RGBDropDown.setCurrentIndex(self.color)
                 elif (event.key() == QtCore.Qt.Key_Comma or
                         event.key() == QtCore.Qt.Key_Period):
                     count = self.BrushChoose.count()
@@ -885,8 +918,10 @@ class MainW(QMainWindow):
         self.opacity = 128 # how opaque masks should be
         self.outcolor = [200,200,255,200]
         self.NZ, self.Ly, self.Lx = 1,512,512
-        if self.autobtn.isChecked():
-            self.saturation = [[0,255] for n in range(self.NZ)]
+        self.saturation = [[0,255] for n in range(self.NZ)]
+        self.slider.setLow(0)
+        self.slider.setHigh(255)
+        self.slider.show()
         self.currentZ = 0
         self.flows = [[],[],[],[],[[]]]
         self.stack = np.zeros((1,self.Ly,self.Lx,3))
@@ -971,12 +1006,16 @@ class MainW(QMainWindow):
         if self.NZ==1:
             self.removed_cell = [self.ismanual[idx-1], self.cellcolors[idx], np.nonzero(cp), np.nonzero(op)]
             self.redo.setEnabled(True)
+            ar, ac = self.removed_cell[2]
+            d = datetime.datetime.now()        
+            self.track_changes.append([d.strftime("%m/%d/%Y, %H:%M:%S"), 'removed mask', [ar,ac]])
         # remove cell from lists
         self.ismanual = np.delete(self.ismanual, idx-1)
         del self.cellcolors[idx]
         del self.zdraw[idx-1]
         self.ncells -= 1
-        print('>>> removed cell %d'%(idx-1))
+        print('GUI_INFO: removed cell %d'%(idx-1))
+        
         if self.ncells==0:
             self.ClearButton.setEnabled(False)
         if self.NZ==1:
@@ -1007,7 +1046,7 @@ class MainW(QMainWindow):
                 color = self.cellcolors[self.prev_selected]
                 self.draw_mask(z, ar, ac, vr, vc, color, idx=self.prev_selected)
             self.remove_cell(self.selected)
-            print('>>> merged two cells')
+            print('GUI_INFO: merged two cells')
             self.update_plot()
             io._save_sets(self)
             self.undo.setEnabled(False)      
@@ -1105,15 +1144,16 @@ class MainW(QMainWindow):
                     # show single channel
                     image = self.stack[self.currentZ][:,:,0]
                 self.img.setImage(image, autoLevels=False, lut=None)
-            elif self.color==1:
+            elif self.color>0 and self.color<4:
+                image = image[:,:,self.color-1]
+                self.img.setImage(image, autoLevels=False, lut=self.cmap[self.color])
+            elif self.color==4:
                 image = image.astype(np.float32).mean(axis=-1).astype(np.uint8)
                 self.img.setImage(image, autoLevels=False, lut=None)
-            elif self.color==2:
+            elif self.color==5:
                 image = image.astype(np.float32).mean(axis=-1).astype(np.uint8)
                 self.img.setImage(image, autoLevels=False, lut=self.cmap[0])
-            elif self.color>2:
-                image = image[:,:,self.color-3]
-                self.img.setImage(image, autoLevels=False, lut=self.cmap[self.color-2])
+            
             self.img.setLevels(self.saturation[self.currentZ])
         else:
             image = np.zeros((self.Ly,self.Lx), np.uint8)
@@ -1129,8 +1169,8 @@ class MainW(QMainWindow):
         #self.img.set_ColorMap(self.bwr)
         if self.masksOn or self.outlinesOn:
             self.layer.setImage(self.layers[self.currentZ], autoLevels=False)
-        #self.slider.setLow(self.saturation[self.currentZ][0])
-        #self.slider.setHigh(self.saturation[self.currentZ][1])
+        self.slider.setLow(self.saturation[self.currentZ][0])
+        self.slider.setHigh(self.saturation[self.currentZ][1])
         self.win.show()
         self.show()
 
@@ -1223,7 +1263,9 @@ class MainW(QMainWindow):
                 ar, ac = ar+ymin, ac+xmin
                 self.draw_mask(z+zmin, ar, ac, vr, vc, color)
         self.zdraw.append(zdraw)
-
+        if self.NZ==1:
+            d = datetime.datetime.now()
+            self.track_changes.append([d.strftime("%m/%d/%Y, %H:%M:%S"), 'added mask', [ar,ac]])
         return median
 
     def draw_mask(self, z, ar, ac, vr, vc, color, idx=None):
@@ -1346,7 +1388,11 @@ class MainW(QMainWindow):
         train = TW.exec_()
         if train:
             logger.info(f'training with {[os.path.split(f)[1] for f in self.train_files]}')
-            self.get_model_path()
+            if self.pretrained_to_use != 'scratch':
+                self.get_model_path()
+            else:
+                self.current_model = 'scratch'
+                self.current_model_path = None
             self.channels = self.get_channels()
             logger.info(f'training with chan (cyto) = {self.ChannelChoose[0].currentText()}, chan2 (nuclei)={self.ChannelChoose[1].currentText()}')
             
@@ -1373,24 +1419,33 @@ class MainW(QMainWindow):
         save_path = os.path.dirname(self.filename)
         d = datetime.datetime.now()
         netstr = self.current_model + d.strftime("_%Y%m%d_%H%M%S")
-        self.new_model_path = self.model.retrain(self.train_data, self.train_labels, self.train_files, 
+        print(netstr)
+        self.new_model_path = self.model.train(self.train_data, self.train_labels, self.train_files, 
                                                  channels=self.channels, save_path=save_path, 
                                                  learning_rate=self.learning_rate, n_epochs=self.n_epochs,
                                                  weight_decay=self.weight_decay, 
+                                                 nimg_per_epoch=8,
                                                  netstr=netstr)
         
         # run model on next image 
-        io._add_model(self, self.new_model_path)
+        io._add_model(self, self.new_model_path, permanent=False)
         self.new_model_ind = len(self.model_strings)-1
         print(f'GUI_INFO: model saved to {self.new_model_path} and loaded in gui')
         self.autorun = True
         if self.autorun:
+            channels = self.channels.copy()
             self.get_next_image(load_seg=True)
+            # keep same channels
+            self.ChannelChoose[0].setCurrentIndex(channels[0])
+            self.ChannelChoose[1].setCurrentIndex(channels[1])
             if self.train_files[0] == self.filename:
                 print(f'GUI_INFO: trained on all images + masks in folder --> auto-end training')
                 self.end_train() 
                 #self.get_next_image(load_seg=True)
                 return    
+            diam_train = np.array([diameters(masks)[0] for masks in self.train_labels])
+            self.diameter = diam_train.mean()
+            self.Diameter.setText('%0.1f'%self.diameter)        
             self.compute_model()
         logger.info(f'!!! computed masks for {os.path.split(self.filename)[1]} from new model !!!')
         
@@ -1418,14 +1473,22 @@ class MainW(QMainWindow):
             thresh = self.threshold
             logger.info('computing masks with cell prob=%0.3f, flow error threshold=%0.3f'%
                     (self.cellprob, thresh))
-        maski = dynamics.compute_masks(self.flows[4][:-1], 
-                                       self.flows[4][-1],
-                                       p=self.flows[3].copy(),
-                                       mask_threshold=self.cellprob,
-                                       flow_threshold=thresh,
-                                       resize=self.cellpix.shape[-2:],
-                                       omni=OMNI_INSTALLED and self.omni.isChecked(),
-                                       cluster=OMNI_INSTALLED and self.cluster.isChecked())[0]
+
+        if not (OMNI_INSTALLED and self.omni.isChecked()):
+            maski = dynamics.compute_masks(self.flows[4][:-1], 
+                                            self.flows[4][-1],
+                                            p=self.flows[3].copy(),
+                                            mask_threshold=self.cellprob,
+                                            flow_threshold=thresh,
+                                            resize=self.cellpix.shape[-2:])[0]
+        else:
+            maski = omnipose.core.compute_masks(self.flows[4][:-1], 
+                                            self.flows[4][-1],
+                                            p=self.flows[3].copy(),
+                                            mask_threshold=self.cellprob,
+                                            flow_threshold=thresh,
+                                            resize=self.cellpix.shape[-2:],
+                                            cluster=self.cluster.isChecked())[0]
         
         self.masksOn = True
         self.MCheckBox.setChecked(True)
@@ -1457,19 +1520,23 @@ class MainW(QMainWindow):
             try:
                 omni_model = 'omni' in self.current_model
                 bacterial = 'bact' in self.current_model
-                if omni_model:
-                    self.NetAvg.setCurrentIndex(2) #one run net
+                if omni_model or bacterial:
+                    self.NetAvg.setCurrentIndex(1) #one run net
                 if bacterial:
                     self.diameter = 0.
                     self.Diameter.setText('%0.1f'%self.diameter)
-                    
+                
                 # allow omni to be togged manually or forced by model
                 if OMNI_INSTALLED:
                     self.omni.setChecked(self.omni.isChecked() or omni_model) 
                     self.cluster.setChecked(self.cluster.isChecked() or omni_model)
-                
-                net_avg = self.NetAvg.currentIndex()<2 and self.current_model in models.MODEL_NAMES
-                resample = self.NetAvg.currentIndex()==1
+                    if omni_model:
+                        print('GUI_INFO: turning on omnipose mask creation version for omnipose models (see menu)')
+                    elif self.omni.isChecked():
+                        print('WARNING: using omnipose mask creation with built-in cellpose model (turn off in Omnipose menu)')
+
+                net_avg = self.NetAvg.currentIndex()==0 and self.current_model in models.MODEL_NAMES
+                resample = self.NetAvg.currentIndex()<2
                 masks, flows = self.model.eval(data, channels=channels,
                                                 diameter=self.diameter, invert=self.invert.isChecked(),
                                                 net_avg=net_avg, augment=False, resample=resample,
@@ -1483,7 +1550,7 @@ class MainW(QMainWindow):
             #if not do_3D:
             #    masks = masks[0][np.newaxis,:,:]
             #    flows = flows[0]
-            self.flows[0] = (np.clip(normalize99(flows[0].copy()), 0, 1) * 255).astype(np.uint8) #RGB flow
+            self.flows[0] = flows[0].copy() #RGB flow
             self.flows[1] = (np.clip(normalize99(flows[2].copy()), 0, 1) * 255).astype(np.uint8) #dist/prob
             if not do_3D:
                 masks = masks[np.newaxis,...]
