@@ -1,4 +1,4 @@
-import os, datetime, gc, warnings, glob, shutil
+import os, datetime, gc, warnings, glob, shutil, copy
 from natsort import natsorted
 import numpy as np
 import cv2
@@ -8,6 +8,7 @@ import fastremap
 
 from .. import utils, plot, transforms, models
 from ..io import imread, imsave, outlines_to_text
+from ..transforms import normalize99
 
 try:
     from PyQt5.QtWidgets import QFileDialog
@@ -37,8 +38,9 @@ def _init_model_list(parent):
             lines = [line.rstrip() for line in textfile]
             if len(lines) > 0:
                 parent.model_strings.extend(lines)
+    parent.permanent_model = [True for i in range(len(parent.model_strings))]
 
-def _add_model(parent, filename=None):
+def _add_model(parent, filename=None, permanent=True):
     if filename is None:
         name = QFileDialog.getOpenFileName(
             parent, "Add model to GUI"
@@ -51,7 +53,13 @@ def _add_model(parent, filename=None):
         textfile.write(fname + '\n')
     parent.ModelChoose.addItems([fname])
     parent.model_strings.append(fname)
+    parent.permanent_model.append(permanent)
     parent.ModelChoose.setCurrentIndex(len(parent.model_strings) - 1)
+    parent.NetAvg.setCurrentIndex(1)
+
+#def _remove_non_permanent_models(parent):
+#    for perm in parent.permanent_model:
+#        if not perm: 
 
 def _remove_model(parent, ind=None):
     if ind is None:
@@ -60,6 +68,7 @@ def _remove_model(parent, ind=None):
         print(f'GUI_INFO: deleting {parent.model_strings[ind]} from GUI')
         parent.ModelChoose.removeItem(ind)
         del parent.model_strings[ind]
+        del parent.permanent_model[ind]
         custom_strings = parent.model_strings[len(models.MODEL_NAMES):]
         if len(custom_strings) > 0:
             with open(parent.model_list_path, 'w') as textfile:
@@ -108,14 +117,18 @@ def _load_image(parent, filename=None, load_seg=True):
             )
         filename = name[0]
     manual_file = os.path.splitext(filename)[0]+'_seg.npy'
+    load_mask = False
     if load_seg:
-        if os.path.isfile(manual_file):
+        if os.path.isfile(manual_file) and not parent.autoloadMasks.isChecked():
             _load_seg(parent, manual_file, image=imread(filename), image_file=filename)
             return
         elif os.path.isfile(os.path.splitext(filename)[0]+'_manual.npy'):
             manual_file = os.path.splitext(filename)[0]+'_manual.npy'
             _load_seg(parent, manual_file, image=imread(filename), image_file=filename)
             return
+        elif parent.autoloadMasks.isChecked():
+            mask_file = os.path.splitext(filename)[0]+'_masks'+os.path.splitext(filename)[-1]
+            load_mask = True if os.path.isfile(mask_file) else False
     try:
         image = imread(filename)
         parent.loaded = True
@@ -130,6 +143,8 @@ def _load_image(parent, filename=None, load_seg=True):
         parent.clear_all()
         parent.loaded = True
         parent.enable_buttons()
+        if load_mask:
+            _load_masks(parent, filename=mask_file)
         parent.threshslider.setEnabled(False)
         parent.probslider.setEnabled(False)
             
@@ -156,15 +171,18 @@ def _initialize_images(parent, image, resize, X2):
     elif image.ndim==3:
         if image.shape[0] < 5:
             image = np.transpose(image, (1,2,0))
-
         if image.shape[-1] < 3:
             shape = image.shape
+            #if parent.autochannelbtn.isChecked():
+            #    image = normalize99(image) * 255
             image = np.concatenate((image,np.zeros((shape[0], shape[1], 3-shape[2]),dtype=type(image[0,0,0]))), axis=-1)
             if 3-shape[2]>1:
                 parent.onechan=True
             image = image[np.newaxis,...]
         elif image.shape[-1]<5 and image.shape[-1]>2:
             image = image[:,:,:3]
+            #if parent.autochannelbtn.isChecked():
+            #    image = normalize99(image) * 255
             image = image[np.newaxis,...]
     else:
         image = image[np.newaxis,...]
@@ -172,11 +190,12 @@ def _initialize_images(parent, image, resize, X2):
     parent.stack = image
     parent.NZ = len(parent.stack)
     parent.scroll.setMaximum(parent.NZ-1)
-    if parent.stack.max()>255 or parent.stack.min()<0.0 or parent.stack.max()<=50.0:
-        parent.stack = parent.stack.astype(np.float32)
-        parent.stack -= parent.stack.min()
-        parent.stack /= parent.stack.max()
-        parent.stack *= 255
+    parent.stack = parent.stack.astype(np.float32)
+    for i in range(parent.stack.shape[-1]):
+        parent.stack[...,i] -= parent.stack[...,i].min()
+        if np.ptp(parent.stack[...,i]) > 1e-6:
+            parent.stack[...,i] /= parent.stack[...,i].max()
+    parent.stack *= 255
     del image
     gc.collect()
 
@@ -202,6 +221,7 @@ def _initialize_images(parent, image, resize, X2):
     parent.currentZ = int(np.floor(parent.NZ/2))
     parent.scroll.setValue(parent.currentZ)
     parent.zpos.setText(str(parent.currentZ))
+    parent.track_changes = []
 
 def _load_seg(parent, filename=None, image=None, image_file=None):
     """ load *_seg.npy with filename; if None, open QFileDialog """
@@ -322,11 +342,11 @@ def _load_seg(parent, filename=None, image=None, image_file=None):
 
     if 'flows' in dat:
         parent.flows = dat['flows']
-        if parent.flows[0].shape[-3]!=dat['masks'].shape[-2]:
-            Ly, Lx = dat['masks'].shape[-2:]
-            parent.flows[0] = cv2.resize(parent.flows[0][0], (Lx, Ly), interpolation=cv2.INTER_NEAREST)[np.newaxis,...]
-            parent.flows[1] = cv2.resize(parent.flows[1][0], (Lx, Ly), interpolation=cv2.INTER_NEAREST)[np.newaxis,...]
         try:
+            if parent.flows[0].shape[-3]!=dat['masks'].shape[-2]:
+                Ly, Lx = dat['masks'].shape[-2:]
+                parent.flows[0] = cv2.resize(parent.flows[0].squeeze(), (Lx, Ly), interpolation=cv2.INTER_NEAREST)[np.newaxis,...]
+                parent.flows[1] = cv2.resize(parent.flows[1].squeeze(), (Lx, Ly), interpolation=cv2.INTER_NEAREST)[np.newaxis,...]
             if parent.NZ==1:
                 parent.threshslider.setEnabled(True)
                 parent.probslider.setEnabled(True)
@@ -461,10 +481,10 @@ def _save_sets(parent):
                  'chan_choose': [parent.ChannelChoose[0].currentIndex(),
                                  parent.ChannelChoose[1].currentIndex()],
                  'img': image.squeeze(),
-                 'ismanual': parent.ismanual,
-                 'X2': parent.X2,
                  'filename': parent.filename,
                  'flows': parent.flows,
+                 'ismanual': parent.ismanual,
+                 'manual_changes': parent.track_changes,
                  'model_path': parent.current_model_path if hasattr(parent, 'current_model_path') else 0})
     #print(parent.point_sets)
     print('GUI_INFO: %d ROIs saved to %s'%(parent.ncells, base + '_seg.npy'))
