@@ -10,12 +10,7 @@ models_logger = logging.getLogger(__name__)
 
 from . import transforms, dynamics, utils, plot
 from .core import UnetModel, assign_device, check_mkl, MXNET_ENABLED, parse_model_string
-
-try:
-    import omnipose
-    OMNI_INSTALLED = True
-except:
-    OMNI_INSTALLED = False
+from .io import OMNI_INSTALLED
 
 _MODEL_URL = 'https://www.cellpose.org/models'
 _MODEL_DIR_ENV = os.environ.get("CELLPOSE_LOCAL_MODELS_PATH")
@@ -90,13 +85,8 @@ class Cellpose():
         if model_type=='cyto2' and not self.torch:
             model_type='cyto'
                 
-        self.omni = omni or 'omni' in model_type       
         
         # for now, omni models cannot do net_avg 
-        if self.omni:
-            net_avg = False
-        model_range = range(4) if net_avg else range(1)
-        self.pretrained_model = [model_path(model_type, j, torch) for j in model_range]
         
         self.diam_mean = 30. #default for any cyto model 
         nuclear = 'nuclei' in model_type
@@ -107,12 +97,19 @@ class Cellpose():
             #self.diam_mean = 0.
             net_avg = False # No bacterial or omni models have additional models
         
+        self.omni = omni or 'omni' in model_type       
+        if self.omni:
+            net_avg = False
+        model_range = range(4) if net_avg else range(1)
+        self.pretrained_model = [model_path(model_type, j, torch) for j in model_range]
+
         if not net_avg:
             self.pretrained_model = self.pretrained_model[0]
 
         self.cp = CellposeModel(device=self.device, gpu=self.gpu,
                                 pretrained_model=self.pretrained_model,
-                                diam_mean=self.diam_mean, torch=self.torch, omni=self.omni)
+                                diam_mean=self.diam_mean, torch=self.torch, omni=self.omni,
+                                net_avg=net_avg)
         self.cp.model_type = model_type
         
         # size model not used for bacterial model
@@ -126,10 +123,10 @@ class Cellpose():
 
     def eval(self, x, batch_size=8, channels=None, channel_axis=None, z_axis=None,
              invert=False, normalize=True, diameter=30., do_3D=False, anisotropy=None,
-             net_avg=True, augment=False, tile=True, tile_overlap=0.1, resample=False, interp=True, cluster=False,
+             net_avg=True, augment=False, tile=True, tile_overlap=0.1, resample=True, interp=True, cluster=False,
              flow_threshold=0.4, mask_threshold=0.0, cellprob_threshold=None, dist_threshold=None,
              diam_threshold=12., min_size=15, stitch_threshold=0.0, 
-             rescale=None, progress=None, omni=False, verbose=False, transparency=False):
+             rescale=None, progress=None, omni=False, verbose=False, transparency=False, model_loaded=False):
         """ run cellpose and get masks
 
         Parameters
@@ -182,7 +179,7 @@ class Cellpose():
         tile_overlap: float (optional, default 0.1)
             fraction of overlap of tiles when computing flows
 
-        resample: bool (optional, default False)
+        resample: bool (optional, default True)
             run dynamics at original image size (will be slower but create more accurate boundaries)
 
         interp: bool (optional, default True)
@@ -228,6 +225,9 @@ class Cellpose():
         transparency: bool (optional, default False)
             modulate flow opacity by magnitude instead of brightness (can use flows on any color background) 
 
+        model_loaded: bool (optional, default False)
+            internal variable for determining if model has been loaded, used in __main__.py
+
         Returns
         -------
         masks: list of 2D arrays, or single 3D array (if do_3D=True)
@@ -237,8 +237,8 @@ class Cellpose():
             flows[k][0] = XY flow in HSV 0-255
             flows[k][1] = flows at each pixel
             flows[k][2] = scalar cell probability (Cellpose) or distance transform (Omnipose)
-            flows[k][3] = boundary output (nonempty for Omnipose)
-            flows[k][4] = final pixel locations after Euler integration 
+            flows[k][3] = final pixel locations after Euler integration 
+            flows[k][4] = boundary output (nonempty for Omnipose)
             flows[k][5] = pixel traces (nonempty for calc_trace=True)
 
         styles: list of 1D arrays of length 256, or single 1D array (if do_3D=True)
@@ -308,7 +308,8 @@ class Cellpose():
                                             stitch_threshold=stitch_threshold,
                                             omni=omni,
                                             verbose=verbose,
-                                            transparency=transparency)
+                                            transparency=transparency,
+                                            model_loaded=model_loaded)
         models_logger.info('>>>> TOTAL TIME %0.2f sec'%(time.time()-tic0))
     
         return masks, flows, styles, diams
@@ -382,9 +383,6 @@ class CellposeModel(UnetModel):
             
             if nuclear:
                 self.diam_mean = 17. 
-            elif bacterial:
-                #self.diam_mean = 0.
-                net_avg = False #'bact' model also has no 1,2,3
 
             # set omni flag to true if the name contains it
             self.omni = 'omni' in os.path.splitext(Path(pretrained_model_string).name)[0]
@@ -417,6 +415,8 @@ class CellposeModel(UnetModel):
         self.pretrained_model = pretrained_model
         if self.pretrained_model and len(self.pretrained_model)==1:
             self.net.load_model(self.pretrained_model[0], cpu=(not self.gpu))
+            if not self.torch:
+                self.net.collect_params().grad_req = 'null'
         ostr = ['off', 'on']
         omnistr = ['','_omni'] #toggle by containing omni phrase 
         self.net_type = 'cellpose_residual_{}_style_{}_concatenation_{}{}'.format(ostr[residual_on],
@@ -428,11 +428,11 @@ class CellposeModel(UnetModel):
              z_axis=None, normalize=True, invert=False, 
              rescale=None, diameter=None, do_3D=False, anisotropy=None, net_avg=True, 
              augment=False, tile=True, tile_overlap=0.1,
-             resample=False, interp=True, cluster=False,
+             resample=True, interp=True, cluster=False,
              flow_threshold=0.4, mask_threshold=0.0, diam_threshold=12.,
              cellprob_threshold=None, dist_threshold=None,
              compute_masks=True, min_size=15, stitch_threshold=0.0, progress=None, omni=False, 
-             calc_trace=False, verbose=False, transparency=False):
+             calc_trace=False, verbose=False, transparency=False, loop_run=False, model_loaded=False):
         """
             segment list of images x, or 4D array - Z x nchan x Y x X
 
@@ -490,7 +490,7 @@ class CellposeModel(UnetModel):
             tile_overlap: float (optional, default 0.1)
                 fraction of overlap of tiles when computing flows
 
-            resample: bool (optional, default False)
+            resample: bool (optional, default True)
                 run dynamics at original image size (will be slower but create more accurate boundaries)
 
             interp: bool (optional, default True)
@@ -533,6 +533,12 @@ class CellposeModel(UnetModel):
             
             transparency: bool (optional, default False)
                 modulate flow opacity by magnitude instead of brightness (can use flows on any color background) 
+            
+            loop_run: bool (optional, default False)
+                internal variable for determining if model has been loaded, stops model loading in loop over images
+
+            model_loaded: bool (optional, default False)
+                internal variable for determining if model has been loaded, used in __main__.py
 
             Returns
             -------
@@ -560,6 +566,7 @@ class CellposeModel(UnetModel):
             if omni:
                 models_logger.info('using omni model, cluster %d'%(omni,cluster))
         
+        
         if isinstance(x, list) or x.squeeze().ndim==5:
             masks, styles, flows = [], [], []
             tqdm_out = utils.TqdmToLogger(models_logger, level=logging.INFO)
@@ -596,13 +603,20 @@ class CellposeModel(UnetModel):
                                                  omni=omni,
                                                  calc_trace=calc_trace, 
                                                  verbose=verbose,
-                                                 transparency=transparency)
+                                                 transparency=transparency,
+                                                 loop_run=(i>0),
+                                                 model_loaded=model_loaded)
                 masks.append(maski)
                 flows.append(flowi)
                 styles.append(stylei)
             return masks, styles, flows 
         
         else:
+            if not model_loaded and (isinstance(self.pretrained_model, list) and not net_avg and not loop_run):
+                self.net.load_model(self.pretrained_model[0], cpu=(not self.gpu))
+                if not self.torch:
+                    self.net.collect_params().grad_req = 'null'
+
             x = transforms.convert_image(x, channels, channel_axis=channel_axis, z_axis=z_axis,
                                          do_3D=(do_3D or stitch_threshold>0), normalize=False, invert=False, nchan=self.nchan, omni=omni)
             if x.ndim < 4:
@@ -611,11 +625,6 @@ class CellposeModel(UnetModel):
             rescale = self.diam_mean / diameter if (rescale is None and (diameter is not None and diameter>0)) else rescale
             rescale = 1.0 if rescale is None else rescale
             
-            if isinstance(self.pretrained_model, list) and not net_avg:
-                self.net.load_model(self.pretrained_model[0], cpu=(not self.gpu))
-                if not self.torch:
-                    self.net.collect_params().grad_req = 'null'
-            
             masks, styles, dP, cellprob, p, bd, tr = self._run_cp(x, 
                                                           compute_masks=compute_masks,
                                                           normalize=normalize,
@@ -638,12 +647,11 @@ class CellposeModel(UnetModel):
                                                           omni=omni,
                                                           calc_trace=calc_trace,
                                                           verbose=verbose)
-
             flows = [plot.dx_to_circ(dP,transparency=transparency), dP, cellprob, p, bd, tr]
             return masks, flows, styles
 
     def _run_cp(self, x, compute_masks=True, normalize=True, invert=False,
-                rescale=1.0, net_avg=True, resample=False,
+                rescale=1.0, net_avg=True, resample=True,
                 augment=False, tile=True, tile_overlap=0.1,
                 mask_threshold=0.0, diam_threshold=12., flow_threshold=0.4, min_size=15,
                 interp=True, cluster=False, anisotropy=1.0, do_3D=False, stitch_threshold=0.0,
@@ -683,7 +691,6 @@ class CellposeModel(UnetModel):
                     img = transforms.normalize_img(img, invert=invert, omni=omni)
                 if rescale != 1.0:
                     img = transforms.resize_image(img, rsz=rescale)
-
                 yf, style = self._run_nets(img, net_avg=net_avg,
                                            augment=augment, tile=tile,
                                            tile_overlap=tile_overlap)
@@ -707,30 +714,30 @@ class CellposeModel(UnetModel):
 
         if compute_masks:
             tic=time.time()
-            niter = 200 if do_3D else (1 / rescale * 200)
+            niter = 200 if (do_3D and not resample) else (1 / rescale * 200)
             if do_3D:
                 masks, p, tr = dynamics.compute_masks(dP, cellprob, bd, niter=niter, mask_threshold=mask_threshold,
-                                                      diam_threshold=diam_threshold,flow_threshold=flow_threshold,
-                                                      interp=interp, cluster=cluster, do_3D=do_3D, min_size=min_size,
-                                                      resize=None, omni=omni, calc_trace=calc_trace, verbose=verbose,
+                                                      diam_threshold=diam_threshold, flow_threshold=flow_threshold,
+                                                      interp=interp, do_3D=do_3D, min_size=min_size,
+                                                      resize=None, verbose=verbose,
                                                       use_gpu=self.gpu, device=self.device, nclasses=self.nclasses)
             else:
                 masks, p, tr = [], [], []
                 resize = [shape[1], shape[2]] if not resample else None
                 for i in iterator:
-                    bdi = bd[i] if bd is not None else None
-                    outputs = dynamics.compute_masks(dP[:,i], cellprob[i], bdi, 
-                                                    niter=niter, 
-                                                    mask_threshold=mask_threshold,
-                                                    flow_threshold=flow_threshold, 
-                                                    diam_threshold=diam_threshold, 
-                                                    interp=interp, cluster=cluster,
-                                                    resize=resize, 
-                                                    omni=omni, calc_trace=calc_trace, 
-                                                    verbose=verbose,
-                                                    use_gpu=self.gpu, 
-                                                    device=self.device, 
-                                                    nclasses=self.nclasses)
+                    if not (omni and OMNI_INSTALLED):
+                        # run cellpose compute_masks
+                        outputs = dynamics.compute_masks(dP[:,i], cellprob[i], niter=niter, mask_threshold=mask_threshold,
+                                                         flow_threshold=flow_threshold, interp=interp, resize=resize, verbose=verbose,
+                                                         use_gpu=self.gpu, device=self.device, nclasses=self.nclasses)
+                    else:
+                        # run omnipose compute_masks
+                        bdi = bd[i] if bd is not None else None
+                        outputs = omnipose.core.compute_masks(dP[:,i], cellprob[i], bdi, niter=niter, mask_threshold=mask_threshold,
+                                                                flow_threshold=flow_threshold, diam_threshold=diam_threshold, 
+                                                                interp=interp, cluster=cluster, resize=resize, 
+                                                                calc_trace=calc_trace, verbose=verbose,
+                                                                use_gpu=self.gpu, device=self.device, nclasses=self.nclasses)
                     masks.append(outputs[0])
                     p.append(outputs[1])
                     tr.append(outputs[2])
@@ -771,11 +778,12 @@ class CellposeModel(UnetModel):
 
     def train(self, train_data, train_labels, train_files=None, 
               test_data=None, test_labels=None, test_files=None,
-              channels=None, normalize=True, pretrained_model=None, 
+              channels=None, normalize=True, 
               save_path=None, save_every=100, save_each=False,
               learning_rate=0.2, n_epochs=500, momentum=0.9, SGD=True,
-              weight_decay=0.00001, batch_size=8, rescale=True, omni=False,
-              netstr=None):
+              weight_decay=0.00001, batch_size=8, nimg_per_epoch=None,
+              rescale=True, min_train_masks=5,
+              omni=False, netstr=None):
 
         """ train network with images train_data 
         
@@ -808,34 +816,39 @@ class CellposeModel(UnetModel):
             normalize: bool (default, True)
                 normalize data so 0.0=1st percentile and 1.0=99th percentile of image intensities in each channel
 
-            pretrained_model: string (default, None)
-                path to pretrained_model to start from, if None it is trained from scratch
-
             save_path: string (default, None)
                 where to save trained model, if None it is not saved
 
             save_every: int (default, 100)
                 save network every [save_every] epochs
 
-            learning_rate: float (default, 0.2)
-                learning rate for training
+            learning_rate: float or list/np.ndarray (default, 0.2)
+                learning rate for training, if list, must be same length as n_epochs
 
             n_epochs: int (default, 500)
                 how many times to go through whole training set during training
 
             weight_decay: float (default, 0.00001)
 
-            SGD: bool (default, True) use SGD as optimization instead of RAdam
+            SGD: bool (default, True) 
+                use SGD as optimization instead of RAdam
 
             batch_size: int (optional, default 8)
                 number of 224x224 patches to run simultaneously on the GPU
                 (can make smaller or bigger depending on GPU memory usage)
 
+            nimg_per_epoch: int (optional, default None)
+                minimum number of images to train on per epoch, 
+                with a small training set (< 8 images) it may help to set to 8
+
             rescale: bool (default, True)
                 whether or not to rescale images to diam_mean during training, 
                 if True it assumes you will fit a size model after training or resize your images accordingly,
                 if False it will try to train the model to be scale-invariant (works worse)
 
+            min_train_masks: int (default, 5)
+                minimum number of masks an image must have to use in training set
+
             netstr: str (default, None)
                 name of network, otherwise saved with name as params + training start time
 
@@ -846,102 +859,29 @@ class CellposeModel(UnetModel):
                                                                                                    test_data, test_labels,
                                                                                                    channels, normalize, omni)
         # check if train_labels have flows
-        train_flows = dynamics.labels_to_flows(train_labels, files=train_files, use_gpu=self.gpu, device=self.device, omni=omni)
+        # if not, flows computed, returned with labels as train_flows[i][0]
+        labels_to_flows = dynamics.labels_to_flows if not (omni and OMNI_INSTALLED) else omnipose.core.labels_to_flows
+        train_flows = labels_to_flows(train_labels, files=train_files, use_gpu=self.gpu, device=self.device)
         if run_test:
-            test_flows = dynamics.labels_to_flows(test_labels, files=test_files, use_gpu=self.gpu, device=self.device)
+            test_flows = labels_to_flows(test_labels, files=test_files, use_gpu=self.gpu, device=self.device)
         else:
             test_flows = None
-        
+
+        nmasks = np.array([label[0].max()-1 for label in train_flows])
+        nremove = (nmasks < min_train_masks).sum()
+        if nremove > 0:
+            models_logger.warning(f'{nremove} train images with number of masks less than min_train_masks ({min_train_masks}), removing from train set')
+            ikeep = np.nonzero(nmasks >= min_train_masks)[0]
+            train_data = [train_data[i] for i in ikeep]
+            train_flows = [train_flows[i] for i in ikeep]
+
         model_path = self._train_net(train_data, train_flows, 
-                                     test_data, test_flows,
-                                     save_path, save_every, save_each,
-                                     learning_rate, n_epochs, momentum, weight_decay, SGD, 
-                                     batch_size, rescale, netstr)
-        self.pretrained_model = model_path
-        return model_path
-
-    def retrain(self, train_data, train_labels, train_files=None, 
-              test_data=None, test_labels=None, test_files=None,
-              channels=None, normalize=True, 
-              save_path=None, save_every=100, save_each=False,
-              learning_rate=0.025, n_epochs=100, momentum=0.9, SGD=True,
-              weight_decay=0.0001, batch_size=8, rescale=True, omni=False, netstr=None):
-
-        """ retrain network with images train_data 
-        
-            Parameters
-            ------------------
-
-            train_data: list of arrays (2D or 3D)
-                images for training
-
-            train_labels: list of arrays (2D or 3D)
-                labels for train_data, where 0=no masks; 1,2,...=mask labels
-                can include flows as additional images
-
-            train_files: list of strings
-                file names for images in train_data (to save flows for future runs)
-
-            test_data: list of arrays (2D or 3D)
-                images for testing
-
-            test_labels: list of arrays (2D or 3D)
-                labels for test_data, where 0=no masks; 1,2,...=mask labels; 
-                can include flows as additional images
-        
-            test_files: list of strings
-                file names for images in test_data (to save flows for future runs)
-
-            channels: list of ints (default, None)
-                channels to use for training
-
-            normalize: bool (default, True)
-                normalize data so 0.0=1st percentile and 1.0=99th percentile of image intensities in each channel
-
-            save_path: string (default, None)
-                where to save trained model, if None it is not saved
-
-            save_every: int (default, 100)
-                save network every [save_every] epochs
-
-            learning_rate: float (default, 0.2)
-                learning rate for training
-
-            n_epochs: int (default, 500)
-                how many times to go through whole training set during training
-
-            weight_decay: float (default, 0.00001)
-
-            SGD: bool (default, True) use SGD as optimization instead of RAdam
-
-            batch_size: int (optional, default 8)
-                number of 224x224 patches to run simultaneously on the GPU
-                (can make smaller or bigger depending on GPU memory usage)
-
-            rescale: bool (default, True)
-                whether or not to rescale images to diam_mean during training, 
-                if True it assumes you will fit a size model after training or resize your images accordingly,
-                if False it will try to train the model to be scale-invariant (works worse)
-
-            netstr: str (default, None)
-                name of network, otherwise saved with name as params + training start time
-
-        """
-        train_data, train_labels, test_data, test_labels, run_test = transforms.reshape_train_test(train_data, train_labels,
-                                                                                                   test_data, test_labels,
-                                                                                                   channels, normalize, omni)
-        # check if train_labels have flows
-        train_flows = dynamics.labels_to_flows(train_labels, files=train_files, use_gpu=self.gpu, device=self.device, omni=omni)
-        if run_test:
-            test_flows = dynamics.labels_to_flows(test_labels, files=test_files, use_gpu=self.gpu, device=self.device)
-        else:
-            test_flows = None
-        
-        model_path = self._train_net(train_data, train_flows, 
-                                     test_data, test_flows,
-                                     save_path, save_every, save_each,
-                                     learning_rate, n_epochs, momentum, weight_decay, SGD, 
-                                     batch_size, rescale, netstr)
+                                     test_data=test_data, test_labels=test_flows,
+                                     save_path=save_path, save_every=save_every, save_each=save_each,
+                                     learning_rate=learning_rate, n_epochs=n_epochs, 
+                                     momentum=momentum, weight_decay=weight_decay, 
+                                     SGD=SGD, batch_size=batch_size, nimg_per_epoch=nimg_per_epoch, 
+                                     rescale=rescale, netstr=netstr)
         self.pretrained_model = model_path
         return model_path
 
@@ -1071,6 +1011,7 @@ class SizeModel():
                               tile=tile,
                               batch_size=batch_size, 
                               net_avg=False,
+                              resample=False,
                               compute_masks=False)[-1]
 
         diam_style = self._size_estimation(np.array(styles))
@@ -1085,6 +1026,7 @@ class SizeModel():
                              tile=tile,
                              batch_size=batch_size, 
                              net_avg=False,
+                             resample=False,
                              rescale =  self.diam_mean / diam_style if self.diam_mean>0 else 1, 
                              #flow_threshold=0,
                              diameter=None,
